#!/usr/bin/python
#coding: utf-8
#(c) 2017 Ramses Carbajal <@RamsesCarbajal>
from . import BaseEndpointTestCase

class OrdersEndpointTestCase(BaseEndpointTestCase):

    def test_01_customer_create(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        customer = self.client.Customer.create(self.customer_object.copy())
        payment_source = customer.payment_sources[0]
        shipping_contact = customer.shipping_contacts[0]

        assert customer.name  == 'James Howlett'
        assert customer.email == 'logan@x-men.org'
        assert customer.phone == '+525511223344'
        assert payment_source.brand == 'VISA' or payment_source.brand == 'MC'
        assert payment_source.last4 == '4242' or payment_source.last4 == '5100'
        assert payment_source.type  == 'card'

        assert shipping_contact.receiver        == "Marvin Fuller"
        assert shipping_contact.between_streets == "Ackerman Crescent"
        assert shipping_contact.address.city    == "Red Deer"

    def test_02_customer_add_token(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        customer_params = self.customer_object.copy()
        del customer_params["payment_sources"]
        customer = self.client.Customer.create(customer_params)
        payment_source = customer.createPaymentSource(self.payment_source_object.copy())

        assert payment_source.brand == "VISA"
        assert payment_source.last4 == "4242"

    def test_03_customer_update_payment_source(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        customer = self.client.Customer.create(self.customer_object.copy())
        payment_source = customer.payment_sources[0]
        payment_source.update(self.update_payment_source_object.copy())
        payment_source = customer.payment_sources[0]
        assert payment_source.brand == 'VISA' or payment_source.brand == 'MC'
        assert payment_source.last4 == '4242' or payment_source.last4 == '5100'

    def test_04_customer_delete_payment_source(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        customer = self.client.Customer.create(self.customer_object.copy())
        payment_source = customer.payment_sources[0]

        payment_source_deleted = payment_source.delete()

        assert payment_source_deleted.deleted == True

        payment_source = customer.payment_sources[0]

        assert payment_source.brand == 'VISA' or payment_source.brand == 'MC'
        assert payment_source.last4 == '4242' or payment_source.last4 == '5100'

    def test_08_customer_add_shipping_contact(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        customer_params = self.customer_object.copy()
        del customer_params["shipping_contacts"]
        customer = self.client.Customer.create(customer_params)
        shipping_contact = customer.createShippingContact(self.shipping_contact_object.copy())

        assert shipping_contact.phone           == "+525511008811"
        assert shipping_contact.receiver        == "Dr. Manhatan"
        assert shipping_contact.between_streets == "some streets"

    def test_09_customer_update_plan(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        customer = self.client.Customer.create(self.customer_object.copy())
        customer.createSubscription({"plan":"gold-plan"})
        customer.subscription.update({"plan":"opal-plan"})
        customer2 = self.client.Customer.find(customer.id)

        assert customer2.subscription.plan_id == "opal-plan"



#!/usr/bin/python
# coding: utf-8
#(c) 2013 Julian Ceballos <@jceb>

import unittest
import random
import conekta


class BaseEndpointTestCase(unittest.TestCase):
    random.seed()

    client = conekta

    card_charge_object = {
        "description": "Grad Stogies: Second Class",
        "amount": 20000,
        "currency": "MXN",
        "reference_id": "9893-cohib_s1_wolf_pack",
        "card": "tok_test_visa_4242",
        "details": {
            "name": "Wolverine",
            "email": "logan.thomas@xmen.org",
            "phone": "403-342-0642",
            "line_items": [{
              "name": "Shades",
              "description": "Imported From Mex.",
              "unit_price": 20000,
              "quantity": 1,
              "sku": "cohb_s1",
              "category": "eyewear"
           }]
        }
    }

    cash_charge_object = {
        "currency": "MXN",
        "amount": 20000,
        "description": "Grad Stogies: Second Class",
        "reference_id": "9893-cohib_s1_wolf_pack",
        "cash": {
            "type": "oxxo"
        },
        "details": {
            "name": "Wolverine",
            "email": "logan.thomas@xmen.org",
            "phone": "403-342-0642",
            "line_items": [{
              "name": "Shades",
              "description": "Imported From Mex.",
              "unit_price": 20000,
              "quantity": 1,
              "sku": "cohb_s1",
              "category": "eyewear"
           }]
        }
    }

    bank_charge_object = {
        "currency": "MXN",
        "amount": 20000,
        "description": "Grad Stogies: Second Class",
        "reference_id": "9893-cohib_s1_wolf_pack",
        "bank": {
            "type": "banorte"
        },
        "details": {
            "name": "Wolverine",
            "email": "logan.thomas@xmen.org",
            "phone": "403-342-0642",
            "line_items": [{
              "name": "Shades",
              "description": "Imported From Mex.",
              "unit_price": 20000,
              "quantity": 1,
              "sku": "cohb_s1",
              "category": "eyewear"
           }]
        }
    }

    spei_charge_object = {
        "currency": "MXN",
        "amount": 20000,
        "description": "Grad Stogies: Second Class",
        "reference_id": "9893-cohib_s1_wolf_pack",
        "bank": {
            "type": "spei"
        },
        "details": {
            "name": "Wolverine",
            "email": "logan.thomas@xmen.org",
            "phone": "403-342-0642",
            "line_items": [{
              "name": "Shades",
              "description": "Imported From Mex.",
              "unit_price": 20000,
              "quantity": 1,
              "sku": "cohb_s1",
              "category": "eyewear"
           }]
        }
    }

    plan_object = {
        'id':'py_test_plan_' + str(random.randint(1, 1000000)),
        'name': 'Python Test Subscription',
        'amount': 10000,
        'currency': 'MXN',
        'interval': 'week',
        'frequency': 3
    }

    customer_object = {
        'name':'James Howlett',
        'email':'logan@x-men.org',
        'phone':'+525511223344',
        'payment_sources':[
          { 'token_id': 'tok_test_visa_4242',
            'type': 'card'},
          { 'token_id': 'tok_test_mastercard_5100',
            'type': 'card'}
        ],
    "shipping_contacts": [{
        "phone": "+525511223344",
        "receiver": "Marvin Fuller",
        "between_streets": "Ackerman Crescent",
        "address": {
            "street1": "250 Alexis St",
            "city": "Red Deer",
            "state": "Alberta",
            "country": "CA",
            "postal_code": "T4N 0B8"
        }

    }]
    }

    payee_object = {
        'name': 'Graydon Creed',
        'email': 'graydon@friendsofhumanity.com',
        'phone': '5555555555',
        'bank': {
            'account_number': '123456789012345673',
            'account_holder': 'Friends of Humanity'
        }
    }

    payout_method_object = {
        'type': 'bank_transfer_payout_method',
        'account_number': '123456789012345673',
        'account_holder': 'Friends of Humanity'
    }

    order_object = {
      "line_items": [
          {
              "name": "Box of Cohiba S1s",
              "description": "Imported From Mex.",
              "unit_price": 20000,
              "quantity": 1,
              "sku": "cohb_s1",
              "category": "food",
              "type" : "physical",
              "tags" : ["food", "mexican food"]
          }
      ],
      "tax_lines":[
        {
          "description": "IVA",
          "amount": 600,
          "metadata": {
            "random_key": "random_value"
          }
        }],
      "shipping_lines":[
        {
          "amount": 0,
          "tracking_number": "TRACK123",
          "carrier": "USPS",
          "method": "Train",
          "metadata": {
             "random_key": "random_value"
          }
        }],
      "discount_lines":[
        {
          "code": "descuento",
          "type": "loyalty",
          "amount": 600
        }],
      "customer_info":{
          "name": "John Constantine",
          "phone": "+525533445566",
          "email": "john@meh.com",
          "corporate": False,
          "vertical_info": {}
        },
      "shipping_contact":{
          "phone" : "5544332211",
          "receiver": "Marvin Fuller",
          "between_streets": "Ackerman Crescent",
          "address": {
              "street1": "250 Alexis St",
              "state": "Alberta",
              "country": "MX",
              "postal_code": "23455",
              "metadata":{ "soft_validations": True}
          }
      },
      "charges": [{
        "payment_method":{
          "type":"oxxo_cash"
        },
        "amount": 20000
      }],
      "currency" : "mxn",
      "metadata" : {"test" : "extra info"}
    }
    line_item_object = {
        "name": "test line item",
        "description": "Imported From Mex.",
        "unit_price": 10000,
        "quantity": 2,
        "sku": "noSKU",
        "category": "food",
        "type" : "physical",
        "tags" : ["food", "mexican food"]
    }
    tax_line_object = {
        "description": "IVA2",
        "amount": 600,
        "metadata": {
        "random_key": "random_value"
        }
    }

    shipping_lines_object = {
        "amount": 0,
        "tracking_number": "TRACK123",
        "carrier": "USPS",
        "method": "Train",
        "metadata": {
           "random_key": "random_value"
        }
    }

    discount_line_object = {
        "code": "descuento",
        "type": "loyalty",
        "amount": 100
    }

    order_shipping_contact_object = {
      "shipping_contact": {
        "phone" : "+525511223399",
        "receiver": "Marvin Fuller",
        "between_streets": "Ackerman Crescent",
        "address": {
            "street1": "250 Alexis St",
            "state": "Alberta",
            "country": "MX",
            "postal_code": "23455",
            "metadata":{ "soft_validations": True}
          }
      }
    }

    charge_object = {
      "payment_method":{
        "type":"oxxo_cash"
      },
      "amount": 20000
    }

    payment_source_object = {
      'token_id': 'tok_test_visa_4242',
      'type': 'card'
    }

    update_payment_source_object = {
        "name": "Emiliano Cabrera",
        "exp_month": "12",
        "exp_year": "20",
        "address": {
            "street1": "Nuevo leon",
            "city": "Monterrey",
            "state": "Nuevo Leon",
            "country": "MX",
            "postal_code": "64700"
        }
    }

    shipping_contact_object = {
        "phone": "+525511008811",
        "receiver": "Dr. Manhatan",
        "between_streets": "some streets",
        "address": {
            "street1": "250 Alexis St",
            "city": "Red Deer",
            "state": "Alberta",
            "country": "CA",
            "postal_code": "T4N 0B8"
        }

    }



#!/usr/bin/python
#coding: utf-8
#(c) 2013 Julian Ceballos <@jceb>

import os
import base64
import inspect
import urllib
import time
import platform
from httplib2 import Http

try:
    import json
except ImportError:
    import simplejson as json

API_VERSION = '2.0.0'

__version__ = '2.3.6'
__author__ = 'Leo Fischer'

API_BASE = 'https://api.conekta.io/'

data = {
    'lang' : 'python',
    'lang_version' : platform.python_version(),
    'publisher' : 'conekta',
    'bindings_version' : __version__,
    'uname' : platform.uname()
}

HEADERS = {
    'Accept': 'application/vnd.conekta-v%s+json' % (API_VERSION),
    'Content-type': 'application/json',
    'X-Conekta-Client-User-Agent' : json.dumps(data)
}

api_key = ''
locale = 'en'

class ConektaError(Exception):
  def __init__(self, error_json):
      super(ConektaError, self).__init__(error_json)
      self.error_json = error_json

class MalformedRequestError(ConektaError): pass
class AuthenticationError(ConektaError): pass
class ProcessingError(ConektaError): pass
class ResourceNotFoundError(ConektaError): pass
class ParameterValidationError(ConektaError): pass
class ApiError(ConektaError): pass

class _Resource(object):
    def __init__(self, attributes):
        self.initialize_instance(attributes)

    def __getitem__(self, key):
        return self.__dict__[key]

    def __setitem__(self, key, value):
        self.__dict__[key] = value

    @classmethod
    def build_http_request(cls, method, path, params, _api_key=None):
        if _api_key is None:
            HEADERS['Authorization'] = 'Basic %s' % (base64.b64encode((api_key + ':').encode('utf-8'))).decode('ascii')
        else:
            HEADERS['Authorization'] = 'Basic %s' % (base64.b64encode((_api_key + ':').encode("utf-8"))).decode('ascii')

        if not locale is None:
            HEADERS['Accept-Language'] = locale

        absolute_url = API_BASE + path
        request = Http(ca_certs=os.path.join(os.path.dirname(__file__), 'ssl_data/ca_bundle.crt')).request
        if method == 'GET':
            if params is None:
                url = absolute_url
            else:
                try:
                    url = "%s?%s" % (absolute_url, urllib.parse.urlencode(params, True))
                except AttributeError:
                    url = "%s?%s" % (absolute_url, urllib.urlencode(params, True))

            headers, body = request(url, method, headers=HEADERS)
        else:
            if params is None:
                HEADERS['Content-type'] = 'application/x-www-form-urlencoded'
                HEADERS['Content-length'] = '0'
                headers, body = request(absolute_url, method, headers=HEADERS, body='')
                del HEADERS['Content-length']
                HEADERS['Content-type'] = 'application/json'
            else:
                headers, body = request(absolute_url, method, headers=HEADERS, body=json.dumps(params))

        try:
            body = str(body, 'utf-8')
        except TypeError:
            body = str(body)

        if headers['status'] == '200' or headers['status'] == '201':
            response_body = json.loads(body)
            return response_body

        if headers['status'] == '400' or headers['status'] == '400':
            raise MalformedRequestError(json.loads(body))
        elif headers['status'] == '401' or headers['status'] == '401':
            raise AuthenticationError(json.loads(body))
        elif headers['status'] == '402' or headers['status'] == '402':
            raise ProcessingError(json.loads(body))
        elif headers['status'] == '404' or headers['status'] == '404':
            raise ResourceNotFoundError(json.loads(body))
        elif headers['status'] == '422' or headers['status'] == '422':
            raise ParameterValidationError(json.loads(body))
        elif headers['status'] == '500' or headers['status'] == '500':
            raise ApiError(json.loads(body))
        else:
            raise ConektaError(json.loads(body))

    @classmethod
    def load_url(cls, path, method='GET', params=None, api_key=None):
        response = cls.build_http_request(method, path, params, _api_key = api_key)
        return response

    @classmethod
    def class_name(cls):
        try:
            return "%s" % urllib.parse.quote_plus(cls.__name__.lower())
        except AttributeError:
            return "%s" % urllib.quote_plus(cls.__name__.lower())

    @classmethod
    def class_url(cls):
        return "%ss" % cls.class_name()

    def instance_url(self):
        return "%s/%s" % (self.class_url(), self.id)

    def initialize_instance(self, attributes):
        if 'id' in attributes.keys():
            self.id = attributes['id']

        existing_keys = self.__dict__.keys()
        new_keys = attributes.keys()

        old_keys = (set(existing_keys) - set(['parent'])) - set(new_keys)
        for key in old_keys:
            self.__dict__[key] = None

        for key in new_keys:
            self.__dict__[key] = attributes[key]


    def load_via_http_request(self, url=None, method='POST', params=None, api_key=None):
        if url is None:
            url = self.instance_url()
            method = 'GET'

        response = self.load_url(url, method=method, params=params, api_key=api_key)

        self.initialize_instance(response)

        return self

class _EventableResource(_Resource):

    def events(self, params={}, api_key=None):
        uri = self.instance_url()
        if hasattr(self, 'parent'):
            uri = "%s/%s" % (self.instance_url(), self.id)

        event = Event.load_url("%s/events" % uri, 'GET', params, api_key=api_key)
        return Event(event)

class _DeletableResource(_Resource):

    def delete(self, params={}, list_to_remove=None, uri=None, api_key=None):
        if uri is None:
            uri = self.instance_url()

            if hasattr(self, 'parent'):
                uri = "%s/%s" % (self.instance_url(), self.id)



        object_reponse = self.load_via_http_request(uri, 'DELETE', {}, api_key=api_key)

        if list_to_remove != None:
            for remove_object in list_to_remove:
                if remove_object.id == self.id:
                    list_to_remove.remove(remove_object)
                    break

        return object_reponse

class _UpdatableResource(_Resource):

    def update(self, params={}, api_key=None):
        uri = self.instance_url()

        if hasattr(self, 'parent') and not isinstance(self, Subscription):
            uri = "%s/%s" % (self.instance_url(), self.id)

        return self.load_via_http_request(uri, 'PUT', params, api_key=api_key)

class _CreatableResource(_Resource):

    @classmethod
    def create(cls, params, api_key=None):
        endpoint = cls.class_url()
        return cls(cls.load_url(endpoint, method='POST', params=params, api_key=api_key))

class _FindableResource(_Resource):

    @classmethod
    def find(cls, _id, api_key=None):
        endpoint = cls.class_url()
        return cls(cls.load_url("%s/%s" % (endpoint, _id), api_key=api_key ))

    @classmethod
    def where(cls, query={}, limit=10, offset=0, sort=[], api_key=None):
        endpoint = cls.class_url()
        query['limit'] = limit

        response = cls.load_url(endpoint, 'GET', query, api_key=api_key)
        pag = Pagination(response)
        data = response["data"]

        pag.data = []
        for obj in data:
            new_obj = None
            if obj["object"] == "customer":
                new_obj = Customer(obj)
                pag.class_name = Customer

            elif obj["object"] == "order":
                new_obj = Order(obj)
                pag.class_name = Order

            elif obj["object"] == "log":
                new_obj = Log(obj)
                pag.class_name = Log

            pag.data.append(new_obj)


        return pag

    #DEPRECATED aliased method, will be removed in next major release
    @classmethod
    def get(cls, _id, api_key=None):
        cls.find(_id, api_key)

class Card(_UpdatableResource, _DeletableResource):

    def instance_url(self):
        return "customers/%s/cards/%s" % (self.parent.id, self.id)

class Charge(_CreatableResource, _FindableResource):

    def instance_url(self):
        return "orders/%s/charges" % (self.parent.id)

    def refund(self, amount=None, api_key=None):
        if amount is None:
            return self.load_via_http_request("%s/refund" % self.instance_url(), api_key=api_key)
        else:
            return self.load_via_http_request("%s/refund" % self.instance_url(), 'POST', {'amount':amount}, api_key=api_key)

    def capture(self, api_key=None):
        return self.load_via_http_request("%s/capture" % self.instance_url(), api_key=api_key)

class Order(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource, _EventableResource):
    def __init__(self, *args, **kwargs):
        super(Order, self).__init__(*args, **kwargs)
        attributes = args[0]
        self.currency = attributes['currency']
        self.line_items = []
        self.tax_lines = []
        self.shipping_lines = []
        self.discount_lines = []
        self.charges = []
        query = {}
        if 'line_items' in attributes.keys():
            endpoint = 'orders/{}/line_items'.format(attributes['id'])
            response = self.load_url(endpoint,'GET',query,api_key=api_key)
            for line_item in response["data"]:
                new_line_item = LineItem(line_item)
                new_line_item.parent = self
                self.line_items.append(new_line_item)

        if 'tax_lines' in attributes.keys():
            for tax_line in attributes['tax_lines']["data"]:
                new_tax_line = TaxLine(tax_line)
                new_tax_line.parent = self
                self.tax_lines.append(new_tax_line)

        if 'shipping_lines' in attributes.keys():
            for shipping_line in attributes['shipping_lines']["data"]:
                new_shipping_line = ShippingLine(shipping_line)
                new_shipping_line.parent = self
                self.shipping_lines.append(new_shipping_line)

        if 'discount_lines' in attributes.keys():
            for discount_line in attributes['discount_lines']["data"]:
                new_discount_line = DiscountLine(discount_line)
                new_discount_line.parent = self
                self.discount_lines.append(new_discount_line)

        if 'customer_info' in attributes.keys():
            self.customer_info = CustomerInfo(attributes['customer_info'])

        if 'shipping_contact' in attributes.keys():
            self.shipping_contact = ShippingContact(attributes['shipping_contact'])

        if 'charges' in attributes.keys():
            for charge in attributes['charges']["data"]:
                payment_method = None
                if charge['payment_method'] is not None:
                    payment_method = PaymentMethod(charge['payment_method'])
                charge = Charge(charge)
                charge.payment_method = payment_method
                self.charges.append(charge)


    def capture(self, params={}, api_key=None):
        order = Order.load_url("%s/capture" % (self.instance_url()), 'PUT', params, api_key=api_key)
        time.sleep(2)
        new_order = Order.find(self.id)
        self.charges = new_order.charges
        self.payment_status = new_order.payment_status
        return new_order

    def refund(self, params={}, api_key=None):
        order_refund = Order.load_url("%s/refund" % (self.instance_url()), 'POST', params, api_key=api_key)
        time.sleep(2)
        new_order = Order.find(self.id)
        self.charges = new_order.charges
        self.payment_status = new_order.payment_status
        return new_order

    def void(self, params={}, api_key=None):
        order_refund = Order.load_url("%s/void" % (self.instance_url()), 'POST', params, api_key=api_key)
        time.sleep(2)
        new_order = Order.find(self.id)
        self.charges = new_order.charges
        self.payment_status = new_order.payment_status
        return new_order

    def charge(self, params, api_key=None):
        charge = Charge(Charge.load_url("%s/charges" % self.instance_url(), 'POST', params, api_key=api_key))
        self.charges.append(charge)
        return charge

    def createShippingContact(self, params, api_key=None):
        orders = self.update(params)
        self.shipping_contact = ShippingContact(orders['shipping_contact'])
        return self.shipping_contact

    def createLineItem(self, params, api_key=None):
        line_item = LineItem(LineItem.load_url("%s/line_items" % self.instance_url(), 'POST', params, api_key=api_key))
        self.line_items.append(line_item)
        return line_item

    def createTaxLine(self, params, api_key=None):
        tax_line = TaxLine(TaxLine.load_url("%s/tax_lines" % self.instance_url(), 'POST', params, api_key=api_key))
        self.tax_lines.append(tax_line)
        return tax_line

    def createShippingLine(self, params, api_key=None):
        shipping_line = ShippingLine(ShippingLine.load_url("%s/shipping_lines" % self.instance_url(), 'POST', params, api_key=api_key))
        self.shipping_lines.append(shipping_line)
        return shipping_line

    def createDiscountLine(self, params, api_key=None):
        discount_line = DiscountLine(DiscountLine.load_url("%s/discount_lines" % self.instance_url(), 'POST', params, api_key=api_key))
        self.discount_lines.append(discount_line)
        return discount_line

class CustomerInfo(_UpdatableResource): pass

class OrderReturns(_UpdatableResource): pass

class PaymentMethod(_UpdatableResource): pass

class Address(_FindableResource): pass

class Customer(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource, _EventableResource):

    def __init__(self, *args, **kwargs):
        super(Customer, self).__init__(*args, **kwargs)

        attributes = args[0]
        self.payment_sources   = []
        self.shipping_contacts = []
        if 'payment_sources' in attributes.keys():
            for payment_source in attributes['payment_sources']['data']:
                new_payment_source = PaymentSource(payment_source)
                new_payment_source.parent = self
                self.payment_sources.append(new_payment_source)

        if 'shipping_contacts' in attributes.keys():
            for shipping_contact in attributes['shipping_contacts']['data']:
                new_shipping_contact = ShippingContact(shipping_contact)
                new_shipping_contact.address = Address(shipping_contact["address"])
                new_shipping_contact.parent = self
                self.shipping_contacts.append(new_shipping_contact)

        if 'subscription' in attributes.keys() and isinstance(attributes['subscription'], dict):
            attributes['subscription']['parent'] = self
            self.subscription = Subscription(attributes['subscription'])
        else:
            self.subscription = None

    def createPaymentSource(self, params, api_key=None):
        pay_src = PaymentSource.load_url("%s/payment_sources" % self.instance_url(), 'POST', params, api_key=api_key)
        payment_source = PaymentSource(pay_src)
        payment_source.parent = self
        self.payment_sources.append(payment_source)
        return payment_source

    def createShippingContact(self, params, api_key=None):
        shipping = PaymentSource.load_url("%s/shipping_contacts" % self.instance_url(), 'POST', params, api_key=api_key)
        shipping_contact = ShippingContact(shipping)
        shipping_contact.parent = self
        self.shipping_contacts.append(shipping_contact)
        return shipping_contact

    def createSubscription(self, params, api_key=None):
        subscription = Subscription(Subscription.load_url("%s/subscription" % self.instance_url(), 'POST', params, api_key=api_key))
        subscription.parent = self
        self.subscription = subscription
        return subscription

    @property
    def default_card(self):
        if self.default_card_id:
            return [card for card in self.cards if card.id == self.default_card_id][0]
        else:
            return None

class Event(_FindableResource): pass

class Log(_FindableResource): pass

class Payee(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource):

    def __init__(self, *args, **kwargs):
        super(Payee, self).__init__(*args, **kwargs)

        attributes = args[0]
        self.payout_methods = []
        if 'payout_methods' in attributes.keys():
            for payout_method in attributes['payout_methods']:
                payout_method['parent'] = self
                self.payout_methods.append(PayoutMethod(payout_method))

    def createPayoutMethod(self, params, api_key=None):
        payout_method = PayoutMethod(PayoutMethod.load_url("%s/payout_methods" % self.instance_url(), 'POST', params, api_key=api_key))
        payout_method.parent = self
        self.payout_methods.append(payout_method)
        return payout_method

    @property
    def default_payout_method(self):
        if self.default_payout_method_id:
            return [payout_method for payout_method in self.payout_methods if payout_method.id == self.default_payout_method_id][0]
        else:
            return None

class Payout(_CreatableResource, _FindableResource): pass
class Pagination(_CreatableResource):
    def next(self):
        if not hasattr(self, 'next_page_url'):
            return None

        raw_params = self.next_page_url.split("?")[1]
        query = {}
        for elemet_params in raw_params.split("&"):
            key_and_param = elemet_params.split("=")
            query[key_and_param[0]] = key_and_param[1]
        return self.class_name.where(query)

    def before(self):
        if not hasattr(self, 'previous_page_url'):
            return None
        raw_params = self.previous_page_url.split("?")[1]
        query = {}
        for elemet_params in raw_params.split("&"):
            key_and_param = elemet_params.split("=")
            query[key_and_param[0]] = key_and_param[1]
        return self.class_name.where(query)

class PayoutMethod(_UpdatableResource, _DeletableResource):

    def instance_url(self):
        return "payees/%s/payout_methods/%s" % (self.parent.id, self.id)

class Plan(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource): pass

class Subscription(_UpdatableResource):

    def instance_url(self):
        return "customers/%s/subscription" % (self.parent.id)

    def pause(self, finish_billing_cycle=False, until=None, api_key=None):
        return self.load_via_http_request("%s/pause" % self.instance_url(), 'POST', {'until': until}, api_key=api_key)

    def resume(self, api_key=None):
        return self.load_via_http_request("%s/resume" % self.instance_url(), 'POST', None, api_key=api_key)

    def cancel(self, finish_billing_cycle=False, api_key=None):
        return self.load_via_http_request("%s/cancel" % self.instance_url(), 'POST', None, api_key=api_key)

    @property
    def card(self):
        return [card for card in self.parent.cards if card.id == self.card_id][0]

    @property
    def plan(self):
        return Plan.retrieve(self.plan_id)

class Webhook(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource): pass

class LineItem(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource, _EventableResource):

    def instance_url(self):
        return "orders/%s/line_items" % (self.parent.id)

    def delete(self, params={}, api_key=None):
        return super(LineItem, self).delete(params, self.parent.line_items)

class TaxLine(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource, _EventableResource):

    def instance_url(self):
        return "orders/%s/tax_lines" % (self.parent.id)

    def delete(self, params={}, api_key=None):
        return super(TaxLine, self).delete(params, self.parent.tax_lines)

class ShippingLine(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource, _EventableResource):

    def instance_url(self):
        return "orders/%s/shipping_lines" % (self.parent.id)

    def delete(self, params={}, api_key=None):
        return super(ShippingLine, self).delete(params, self.parent.shipping_lines)

class DiscountLine(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource, _EventableResource):

    def instance_url(self):
        return "orders/%s/discount_lines" % (self.parent.id)

    def delete(self, params={}, api_key=None):
        return super(DiscountLine, self).delete(params, self.parent.discount_lines)

class PaymentSource(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource):
    def instance_url(self):
        return "customers/%s/payment_sources" % (self.parent.id)

    def delete(self, params={}, api_key=None):
        return super(PaymentSource, self).delete(params, self.parent.payment_sources)

    def events(self, params={}, api_key=None):
        uri = "%s/payment_sources/%s/events" % (self.parent.instance_url(), self.id)
        event = Event.load_url(uri, 'GET', params, api_key=api_key)
        return Event(event)

class ShippingContact(_CreatableResource, _UpdatableResource, _DeletableResource, _FindableResource):

    def instance_url(self):
        return "customers/%s/shipping_contacts" % (self.parent.id)

    def update(self, params={}, api_key=None):
        uri = "%s/shipping_contacts/%s" % (self.parent.instance_url(), self.id)
        return self.load_via_http_request(uri, 'PUT', params, api_key=api_key)

    def delete(self, params={}, api_key=None):
        uri = "%s/shipping_contacts/%s" % (self.parent.instance_url(), self.id)
        return super(ShippingContact, self).delete(params, self.parent.shipping_contacts, uri)

    def events(self, params={}, api_key=None):
        uri = "%s/shipping_contacts/%s/events" % (self.parent.instance_url(), self.id)
        return Event(Event.load_url(uri, 'GET', params, api_key=api_key))



#!/usr/bin/python
#coding: utf-8
#(c) 2017 Ramses Carbajal <@RamsesCarbajal>

from . import BaseEndpointTestCase

class OrdersEndpointTestCase(BaseEndpointTestCase):

    def test_01_customer_all(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        customer1 = self.client.Customer.create(self.customer_object.copy())
        customer2 = self.client.Customer.create(self.customer_object.copy())
        customer3 = self.client.Customer.create(self.customer_object.copy())
        
        customers = self.client.Customer.where()
        assert customers.data[0].id    

    def test_02_customer_pagination_next(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        customer1 = self.client.Customer.create(self.customer_object.copy())
        customer2 = self.client.Customer.create(self.customer_object.copy())
        customer3 = self.client.Customer.create(self.customer_object.copy())
        query = {}
        query["next"] = customer1.id
        customers = self.client.Customer.where(query)
        new_search = customers.next()

        custs = new_search.data
        assert new_search.data[0].id     

    def test_03_customer_pagination_before(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        customer1 = self.client.Customer.create(self.customer_object.copy())
        customer2 = self.client.Customer.create(self.customer_object.copy())
        customer3 = self.client.Customer.create(self.customer_object.copy())
        query = {}
        query["previous"] = customer1.id
        customers = self.client.Customer.where(query)
        new_search = customers.next()
        
        custs = new_search.data
        assert custs[0].id

    def test_04_order_all(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        self.client.Order.create(self.order_object.copy())
        self.client.Order.create(self.order_object.copy())
        self.client.Order.create(self.order_object.copy())
        
        orders = self.client.Order.where()
        assert orders.data[0].id

    def test_05_order_pagination_next(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        order1 = self.client.Order.create(self.order_object.copy())
        order2 = self.client.Order.create(self.order_object.copy())
        order3 = self.client.Order.create(self.order_object.copy())
        query = {}
        query["next"] = order1.id
        orders = self.client.Order.where(query)
        new_search = orders.next()

        orders = new_search.data
        assert new_search.data[0].id     

    def test_05_order_pagination_before(self):
        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        order1 = self.client.Order.create(self.order_object.copy())
        order2 = self.client.Order.create(self.order_object.copy())
        order3 = self.client.Order.create(self.order_object.copy())
        query = {}
        query["previous"] = order1.id
        orders = self.client.Order.where(query)
        new_search = orders.next()

        orders = new_search.data
        assert new_search.data[0].id    

        


#!/usr/bin/python
#coding: utf-8
#(c) 2016- Leo Fischer / Conekta <@leofischer/@conektaio>

from . import BaseEndpointTestCase

class OrdersEndpointTestCase(BaseEndpointTestCase):
    def test_1_webhooks(self):
        url = 'https://www.example.com'
        url2 = url + '/new_endpoint' 

        self.client.api_key = 'key_ZLy4aP2szht1HqzkCezDEA'
        webhook = self.client.Webhook.create({'url':url})
        assert webhook.url == url
        webhook.update({'url':url2})
        assert webhook.url == url2
        webhook.delete()
        assert webhook.deleted
